@@ -47,6 +47,8 @@ sub compareTo {
 #
 #   volume
 #	Parent WARC::Volume object
+#   collection (optional)
+#	Parent WARC::Collection object, if record found via a collection
 #   offset
 #	Offset of start-of-record within parent volume
 #   compression
@@ -59,6 +61,18 @@ sub compareTo {
 #	Size of uncompressed data block according to "sl" gzip extension
 #   protocol
 #	WARC version found at start of record
+#   logical (optional)
+#	Weak reference to logical record object containing this segment
+#	  (Defined by this class, but only set by WARC::Record::Logical.)
+
+#  Keys tested by logical record heuristics:
+#
+#   compression
+#	defined iff record is compressed
+#   sl_packed_size
+#	defined iff compressed record can be skipped without reading data block
+
+sub DESTROY { our $_total_destroyed;	$_total_destroyed++ }
 
 sub _dbg_dump {
   my $self = shift;
@@ -97,10 +111,16 @@ sub _read {
   my $volume = shift;
   my $offset = shift;
 
-  my %ob = (volume => $volume, offset => $offset);
+  my $handle;
+  if (ref $offset) {		# I/O handle passed in instead
+    $handle = $offset;
+    $offset = tell $handle;
+  } else {			# open new handle and seek to offset
+    $handle = $volume->open;
+    seek $handle, $offset, SEEK_SET or die "seek: $!";
+  }
 
-  my $handle = $volume->open;
-  seek $handle, $offset, SEEK_SET or die "seek: $!";
+  my %ob = (volume => $volume, offset => $offset);
 
   my $magic; my $protocol = '';
   defined(read $handle, $magic, 6) or die "read: $!";
@@ -152,6 +172,8 @@ sub _read {
 
   close $handle;
 
+  { our $_total_read;	$_total_read++ }
+
   bless \%ob, $class;
 }
 
@@ -161,6 +183,21 @@ sub volume { (shift)->{volume} }
 
 sub offset { (shift)->{offset} }
 
+sub logical {
+  my $self = shift;
+
+  my $segment_header_value = $self->field('WARC-Segment-Number');
+  if (defined $self->{logical}) {
+    return $self->{logical};	# cached object remains valid ==> return it
+  } elsif (defined $segment_header_value) {
+    return _read WARC::Record::Logical $self;
+  } else {
+    return $self;		# no continuation records present
+  }
+}
+
+sub segments { return shift }
+
 sub next {
   my $self = shift;
 
@@ -176,7 +213,7 @@ sub next {
 
     if (length $isize > 0	# read off the end yields nothing
 	and $self->{sl_full_size} == unpack 'V', $isize) { # ... and looks valid
-      $next = _read WARC::Record::FromVolume $self->volume, tell $handle;
+      $next = _read WARC::Record::FromVolume $self->volume, $handle;
       close $handle;
       return $next;
     } else {
@@ -221,6 +258,8 @@ sub open_block {
   return $xhandle;
 }
 
+sub open_continued { (shift)->logical->open_block }
+
 sub replay {
   die "not yet implemented"
 }
